@@ -8,11 +8,16 @@ from .shape import *
 from .utils import *
 import subprocess
 import os
+import time
+#import matplotlib.pylab as plt
 
 class Mesher(object):
 
  def __init__(self,argv):
   argv['base'] = np.array(argv['base'])
+
+  self.add_symmetry(argv)
+
   if argv.setdefault('model','lattice') == 'lattice':   
    #create polygons-----
    base = argv.setdefault('base',[[0,0]])
@@ -22,9 +27,13 @@ class Mesher(object):
      return self.generate_bulk_2D(argv)
     else :
      return self.generate_bulk_3D(argv)
-   shape = get_shape(argv)
 
-   polygons = [translate_shape(shape,i) for i in base]
+   a = time.time()
+   shapes = get_shape(argv)
+
+
+   polygons = [translate_shape(shapes[n],i) for n,i in enumerate(base)]
+
 
    argv.update({'polygons':np.array(polygons)})
 
@@ -186,7 +195,27 @@ class Mesher(object):
   with open(os.devnull, 'w') as devnull:
     output = subprocess.check_output("gmsh -optimize_netgen -format msh2 -3 mesh.geo -o mesh.msh".split(), stderr=devnull)
 
- 
+
+ def add_symmetry(self,argv):
+
+  if argv.setdefault('invariance',False):
+      argv['base'] =  np.append(argv['base'],[[0,0]],axis=0)
+
+  if argv.setdefault('reflection',False):
+      
+      base = argv['base'].copy()
+      tmp = base/2 + np.array([0.25,0.25])
+      base = argv['base'].copy()
+      tmp =  np.append(tmp,-base/2 - [0.25,0.25],axis=0)
+      base = argv['base'].copy()
+      base[:,1] = -base[:,1]
+      tmp =  np.append(tmp,base/2 - [-0.25,0.25],axis=0)
+      base = argv['base'].copy()
+      base[:,0] = -base[:,0]
+      tmp =  np.append(tmp,base/2 - [0.25,-0.25],axis=0)
+      argv['base'] = tmp
+
+
 
  def generate_bulk_2D(self,argv):
 
@@ -329,8 +358,14 @@ class Mesher(object):
   pore_wall = []
   delta = 1e-2
   #-------------------------
-  #---
 
+
+  #for pore in polygons:
+  #    for p in pore:
+  #      plt.scatter(p[0],p[1])  
+  #plt.show()
+
+  
   bulk = Frame.difference(cascaded_union(polypores))
   if not (isinstance(bulk, shapely.geometry.multipolygon.MultiPolygon)):
    bulk = [bulk]
@@ -342,6 +377,13 @@ class Mesher(object):
    
     pp = list(region.exterior.coords)[:-1]
     line_list = self.create_line_list(pp,points,lines,store)
+
+    #for l in lines:
+    #  p1 = points[l[0]]
+    #  p2 = points[l[1]]
+    #  plt.plot([p1[0],p2[0]],[p1[1],p2[1]])
+    #plt.show()
+    #quit()
     loops +=1
     local_loops = [loops]
     self.create_loop(loops,line_list,store)
@@ -843,7 +885,8 @@ class Mesher(object):
 
 
    with open(os.devnull, 'w') as devnull:
-    output = subprocess.check_output("gmsh -format msh2 -3 mesh.geo -o mesh.msh".split(), stderr=devnull)
+    output = subprocess.check_output("gmsh -optimize_netgen -format msh2 -3 mesh.geo -o mesh.msh".split(), stderr=devnull)
+    #output = subprocess.check_output("gmsh -format msh2 -3 mesh.geo -o mesh.msh".split(), stderr=devnull)
 
 
  def line_exists_ordered(self,p1,p2):
