@@ -79,7 +79,8 @@ def generate_mfp(**argv):
     for p in range(n_phi):
      angular_average += np.einsum('i,j->ij',direction_ave[t,p],direction_ave[t,p])*domega[t,p]/4.0/np.pi
 
-   rhs_average = np.einsum('m,ij->mij',mfp_sampled*mfp_sampled,angular_average)
+   #rhs_average = np.einsum('m,ij->mij',mfp_sampled*mfp_sampled,angular_average)
+   rhs_average = mfp_sampled*mfp_sampled/3
    #-----------------------------
 
    #replicate bulk values---
@@ -93,14 +94,14 @@ def generate_mfp(**argv):
 
    tc = temp_coeff/np.sum(temp_coeff)
 
-   return {'temp':tc,\
-           'B':[],\
-           'F':direction,\
-           'G':kappa_directional,\
+
+  
+
+   return {'tc':tc,\
+           'VMFP':direction,\
+           'sigma':kappa_directional,\
            'kappa':kappa,\
-           'scale':np.ones((n_mfp,n_theta*n_phi)),\
-           'ac':tc,\
-           'mfp_average':rhs_average,\
+           'mfp_average':rhs_average*1e18,\
            'mfp_sampled':mfp_sampled,\
            'suppression':suppression,\
            'kappam':kappa_bulk,\
