@@ -2,7 +2,6 @@ import warnings
 warnings.simplefilter(action='ignore', category=FutureWarning)
 from pyvtk import *
 import numpy as np
-import deepdish as dd
 import os
 from .utils import *
 import deepdish as dd
@@ -23,7 +22,7 @@ class Plot(object):
    else: 
     if os.path.isfile('geometry.h5') :   
      self.mesh = dd.io.load('geometry.h5')
-
+   
    if 'solver' in argv.keys():
     self.solver = argv['solver'].state
    else: 
@@ -46,7 +45,36 @@ class Plot(object):
     self.plot_maps(**argv)
 
    elif model == 'vtk':
-    self.write_vtk(**argv)   
+    self.write_cell_vtk(**argv)   
+
+ def write_cell_vtk(self,**argv):
+
+   output = []
+   strc = 'CellData('
+   for n,(key, value) in enumerate(self.solver['variables'].items()):
+
+     if  value['data'].shape[0] == 1:  value['data'] =  value['data'][0] #band-aid solution
+     output.append(value['data'])
+     name = value['name'] + '[' + value['units'] + ']'
+     if value['data'].ndim == 1:
+       strc += r'''Scalars(output[''' + str(n) + r'''],name =' ''' + name +  r''' ')'''
+    
+     if value['data'].ndim == 2:
+       strc += r'''Vectors(output[''' + str(n) + r'''],name =' ''' + name +  r''' ')'''
+
+     if n == len(self.solver['variables'])-1:
+      strc += ')'
+     else:
+      strc += ','
+   data = eval(strc)
+
+   if self.mesh['dim'] == 3:
+    vtk = VtkData(UnstructuredGrid(self.mesh['nodes'],tetra=self.mesh['elems']),data)
+   else :
+    vtk = VtkData(UnstructuredGrid(self.mesh['nodes'],triangle=self.mesh['elems']),data)
+
+   vtk.tofile('output.vtk','ascii')
+
 
 
  def write_vtk(self,**argv):
@@ -57,7 +85,8 @@ class Plot(object):
    output = []
    strc = 'PointData('
    for n,(key, value) in enumerate(self.solver['variables'].items()):
-       
+      
+     if  value['data'].shape[0] == 1:  value['data'] =  value['data'][0] #band-aid solution
      output.append(value['data'])
      name = value['name'] + '[' + value['units'] + ']'
      if value['data'].ndim == 1:
@@ -84,7 +113,7 @@ class Plot(object):
 
 
    if data.ndim > 1:
-       node_data = np.zeros((len(self.mesh['nodes']),3))
+       node_data = np.zeros((len(self.mesh['nodes']),int(self.mesh['meta'][2])))
    else:   
        node_data = np.zeros(len(self.mesh['nodes']))
 
@@ -102,9 +131,9 @@ class Plot(object):
 
      triangles = []
      nodes = []
-     for l in list(self.mesh['side_list']['Boundary']) + \
-               list(self.mesh['side_list']['Periodic']) + \
-               list(self.mesh['side_list']['Inactive']) :
+     for l in list(self.mesh['boundary_sides']) + \
+               list(self.mesh['periodic_sides']) + \
+               list(self.mesh['inactive_sides']) :
           tmp = []
           for i in self.mesh['sides'][l]:
               #if i in nodes:  
@@ -132,7 +161,6 @@ class Plot(object):
 
  def plot_maps(self,**argv):
 
-
    if self.mesh['dim'] == 3:
      elems,indices = self.get_surface_nodes() 
    else:  
@@ -141,9 +169,14 @@ class Plot(object):
 
    for key in self.solver['variables'].keys(): 
        self.solver['variables'][key]['data'] = self._get_node_data(self.solver['variables'][key]['data'],indices=indices)
-
    self.solver['variables'][-1] = {'name':'Structure','units':'','data':np.zeros(len(indices))}
 
-   plot_results(self.solver['variables'],self.mesh['nodes'][indices],elems,**argv)
+
+   if argv.setdefault('save_plot',False):
+    dd.io.save('plot.h5',{'variables':self.solver['variables'],'nodes':self.mesh['nodes'][indices],'elems':elems})
+
+   if argv.setdefault('show',True):
+     plot_results(self.solver['variables'],self.mesh['nodes'][indices],elems,**argv)
+
 
 
