@@ -1,4 +1,4 @@
-#import h5py
+from mpi4py import MPI
 from google_drive_downloader import GoogleDriveDownloader as gdd
 import numpy as np
 from shapely.geometry import Polygon
@@ -7,7 +7,7 @@ import shapely
 from shapely.ops import cascaded_union
 import os
 import functools
-
+comm = MPI.COMM_WORLD
 
 def periodic_kernel(x1, x2, p,l,variance):
     return variance*np.exp(-2/l/l * np.sin(np.pi*abs(x1-x2)/p) ** 2)
@@ -92,7 +92,8 @@ def repeat_merge_scale(argv):
        for t in tmp:
         final.append(t)
       else:
-        final.append(thin)
+        #final.append(thin)
+        final.append(p1)
 
   #Create bulk surface---get only the exterior to avoid holes
   MP = MultiPolygon(final) 
@@ -107,7 +108,6 @@ def repeat_merge_scale(argv):
        polygons_final.append(list(conso.exterior.coords))
   
 
-  
 
   #cut redundant points (probably unnecessary)
   new_poly = []
@@ -216,3 +216,88 @@ def get_linear_indexes(mfp,value,scale,extent):
      aj *=mfp[j]/value
    return i,ai,j,aj  
 
+def shared_array(value):
+
+    data = {'dummy':value}
+    return create_shared_memory_dict(data)['dummy']
+
+
+            
+
+
+def create_shared_memory_dict(varss):
+
+       dict_output = {}
+       if comm.Get_rank() == 0:
+          var_meta = {} 
+          for var,value in varss.items():
+
+           if type(value) == list: value = np.array(value)   
+           
+           if value.dtype == np.int64:
+              data_type = 0
+              itemsize = MPI.INT.Get_size()
+           elif value.dtype == np.float64:
+              data_type = 1
+              itemsize = MPI.DOUBLE.Get_size() 
+           else:
+              print('data type for shared memory not supported')
+              quit()
+
+           size = np.prod(value.shape)
+           nbytes = size * itemsize
+           var_meta[var] = [value.shape,data_type,itemsize,nbytes]
+
+       else: nbytes = 0; var_meta = None
+       var_meta = comm.bcast(var_meta,root=0)
+
+       #ALLOCATING MEMORY---------------
+       for n,(var,meta) in enumerate(var_meta.items()):
+        win = MPI.Win.Allocate_shared(meta[3],meta[2], comm=comm) 
+        buf,itemsize = win.Shared_query(0)
+        assert itemsize == meta[2]
+        dt = 'i' if meta[1] == 0 else 'd'
+        output = np.ndarray(buffer=buf,dtype=dt,shape=meta[0]) 
+        if comm.rank == 0: output[:] = varss[var]
+        dict_output[var] = output
+
+       del varss
+       comm.Barrier()
+       return dict_output
+
+
+        
+
+def create_shared_memory(varss):
+       for var in varss:
+         #--------------------------------------
+         if comm.Get_rank() == 0: 
+          tmp = eval('self.' + var)
+          if tmp.dtype == np.int64:
+              data_type = 0
+              itemsize = MPI.INT.Get_size() 
+          elif tmp.dtype == np.float64:
+              data_type = 1
+              itemsize = MPI.DOUBLE.Get_size() 
+          else:
+              print('data type for shared memory not supported')
+              quit()
+          size = np.prod(tmp.shape)
+          nbytes = size * itemsize
+          meta = [tmp.shape,data_type,itemsize]
+         else: nbytes = 0; meta = None
+         meta = comm.bcast(meta,root=0)
+
+         #ALLOCATING MEMORY---------------
+         win = MPI.Win.Allocate_shared(nbytes,meta[2], comm=comm) 
+         buf,itemsize = win.Shared_query(0)
+         assert itemsize == meta[2]
+         dt = 'i' if meta[1] == 0 else 'd'
+         output = np.ndarray(buffer=buf,dtype=dt,shape=meta[0]) 
+
+         if comm.rank == 0:
+             output[:] = tmp  
+
+         exec('self.' + var + '=output')
+
+
