@@ -1,10 +1,10 @@
 import plotly.offline as py
 import plotly.graph_objs as go
 import numpy as np
-import plotly.io as pio
-py.renderer='jupyterlab'
 
 
+py.renderer='jupyterlab'
+
 def plotly_trisurf(nodes, simplices, data,name,units,visible=False):
 
 
@@ -81,13 +81,8 @@ def plot_results(data,nodes,elems,**argv):
         'xanchor': 'center',
         'yanchor': 'top'})
 
-   fig.update_layout(
-    width=500,
-    height=500,
-    autosize=True,
-    margin=dict(t=50, b=20, l=20, r=20),
-    template='plotly_dark'
-   )  
+   fig.update_layout(width=600,height=600,autosize=True,margin=dict(t=50, b=20, l=20, r=20),template='plotly_dark')  
+
 
    updatemenus=[dict(direction='down',active=nv-2 if nv > 1 else 0,x=0.4,y=1.15,buttons=list(buttons),showactive=True)]
    fig.update_layout(updatemenus=updatemenus)
@@ -113,7 +108,6 @@ def plot_results(data,nodes,elems,**argv):
     yaxis=dict(axis),
     zaxis=dict(axis))
 
-
    #camera
    camera = dict(
     up=dict(x=0, y=1, z=0),
@@ -123,10 +117,5 @@ def plot_results(data,nodes,elems,**argv):
 
    fig.update_layout(xaxis_showgrid=False, yaxis_showgrid=False)
    fig.update_layout(scene_camera=camera)
-   #show_in_window(fig)
-
-   #fig.show()
-   #py.plot(fig, filename='OpenBTE.html',auto_open=False)
-   py.plot(fig, filename=argv.setdefault('html_file','index.html'),auto_open =argv.setdefault('auto_open',True))
-   #py_online.plot(fig, filename='OpenBTE',sharing='public')
 
+   py.iplot(fig, filename='OpenBTE')
